
#include <iostream>
#include <map>
#include <cstdlib>
#include <vector>
#include <set>
#include <algorithm>
#include <cmath>
using namespace std;

#define EPS (1E-8)
#define EQ(x,y) (fabs((x)-(y))<EPS)

map<pair<int,int>,int > children2id;
vector<pair<int,int> >  id2children;

vector<double> lrscores;
vector<set<int> > subtrees;
map<pair<int,int>,bool> mem_lt;


bool async_lt(int id1,int id2){
	if(id1==id2) return false;
	pair<int,int> p = make_pair(id1,id2);
	if(mem_lt.find(p) != mem_lt.end())
		return mem_lt[p];
	bool ans;
	if(!EQ(lrscores[id1],lrscores[id2])){
		ans = lrscores[id1]>lrscores[id2];
	}else if(id2children[id1].first != id2children[id2].first){
		ans = async_lt(id2children[id1].first, id2children[id2].first);
	}else if(id2children[id1].second != id2children[id2].second){
		ans = async_lt(id2children[id1].second, id2children[id2].second);
	}else{
		ans = false;
	}
	return mem_lt[p] = ans;
}

int create(){
	char c;
	cin >> c;
	if(c=='0') exit(0);
	if(c=='x') return 0;
	int id1 = create();
	int id2 = create();
	cin >> c;
	if(async_lt(id1,id2)) swap(id1,id2);
	pair<int,int> children = make_pair(id1,id2);
	if(children2id.find(children)!=children2id.end())
		return children2id[children];
	int newid = children2id.size();
	children2id[children] = newid;
	id2children.push_back(children);
	
	set<int> uni,inter;
	set_union(subtrees[id1].begin(), subtrees[id1].end(),
						subtrees[id2].begin(), subtrees[id2].end(),
						inserter(uni, uni.begin()));
	set_intersection(subtrees[id1].begin(), subtrees[id1].end(),
									 subtrees[id2].begin(), subtrees[id2].end(),
									 inserter(inter, inter.begin()));
	lrscores.push_back(1.0*inter.size()/uni.size());
	uni.insert(newid);
	subtrees.push_back(uni);
	return newid;
}

void print(int id, bool left){
	if(id == 0){
		cout << "x";
		return;
	}
	int id1 = id2children[id].first;
	int id2 = id2children[id].second;
	if(!left)
		swap(id1,id2);
	cout << "(";
	print(id1, true);
	cout << " ";
	print(id2, false);
	cout << ")";
}

int main(void){
	while(true){
		children2id.clear();
		children2id[make_pair(-1,-1)]=0;
		id2children.clear();
		id2children.push_back(make_pair(-1,-1));
		lrscores.clear();
		lrscores.push_back(0.0);
		subtrees.clear();
		subtrees.resize(1);
		subtrees[0].insert(0);
		mem_lt.clear();
		int id = create();
		print(id, true);
		cout << endl;
	}
	return 0;
}
